/*
 * Program to convert old format tdb to new format.
 */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#include "tdb.h"

static struct {
    int count;
    int key_bytes;
    int data_bytes;
} add_stats;

static int add_record ( TDB_CONTEXT *tdb, TDB_DATA key, TDB_DATA value, 
	void *dst_vp )
{
    TDB_CONTEXT *dst;
    int status;

    dst = (TDB_CONTEXT *) dst_vp;
    add_stats.count++;
    status = tdb_store ( dst, key, value, TDB_INSERT );
    if ( status != 0 ) {
	printf ( "error inserting record #%d\n", add_stats.count );
	printf ( "Add failure!\n" );
	status = 0;
    } else {
	add_stats.key_bytes += key.dsize;
	add_stats.data_bytes += value.dsize;
    }
    return status;
}

int main ( int argc, char **argv ) 
{
    TDB_DATA key,value;
    TDB_CONTEXT *src, *dst;
    int count;

    if ( argc < 3 ) {
	printf ( "usage: convert_tdb src-file dst-file\n" );
	return 0;
    }
    src = tdb_open ( argv[1], 0, 0, O_RDONLY, 0600 );
    if ( !src ) {
	printf ( "failed to open source file '%s'\n", argv[1] );
	return 0;
    }
    dst = tdb_open ( argv[2], 0, 0, O_RDWR|O_CREAT, 0600 );
    if ( !dst ) {
	printf ( "failed to open destination file '%s'\n", argv[2] );
	return 0;
    }
    /*
     * Read all keys and output to new file.
     */
    add_stats.count = 0;
    add_stats.key_bytes = 0;
    add_stats.data_bytes = 0;
    count = tdb_traverse ( src, add_record, dst );

    printf ( "Copied %d records, key_bytes=%d, data_bytes=%d\n",
	count, add_stats.key_bytes, add_stats.data_bytes );
    return 0;
}

void smb_panic  ( const char *why )
{
    fprintf ( stderr, "PANIC: %s\n", why );
    abort();
}
